/**********************************************************************/
/*                                                                    */
/* This program reads CALIPSO data, Total_Attenuated_Backscatter_532. */
/*                                                                    */
/**********************************************************************/

#include "cloud_sat_2B.v3.h"

/* -------------------------------------------------------------------
   ------------------------------------------------------------------- */
main( int argc, char **argv ) 
{
  initialization( );

  read_year_month_day( argc, argv );

  read_sd_data( argc, argv );
  make_arrays( );
  read_v_data(  argc, argv );

  calc_data( );

  print_data( );

  free_arrays( );
}


/* -------------------------------------------------------------------
   ------------------------------------------------------------------- */
void initialization( )
{
}

/* -------------------------------------------------------------------
   ------------------------------------------------------------------- */
void read_year_month_day( int argc, char **argv )
{
  char c1, c2, c3, c4;
  char y1[4], y2, y3[1], *y4, d1, d2, d3;
  int year, day;
 
  sscanf( argv[1], "%*5c%4d%3d%2d%2d%2d", 
	  &Year, &Dayofyear, &Hour_start, &Minute_start, &Sec_start);

  if( Year == 2006 || Year == 2007 || Year == 2008 || Year == 2009 ){
  }else{
    printf("!Year\n");
    exit( 1 );
  }

  dayofyear_2_month_day( Dayofyear, Year, &Month, &Date );
}

/* ------------------------------------------------------------------------
   ------------------------------------------------------------------------ */
void dayofyear_2_month_day( int dayofyear, int year, int *month, int *date )
{
  int leap_day;

  if( year%4 == 0 && year%100 != 0 && year%400 != 0 ){
    leap_day = 1;
  }else if( year%4 == 0 && year%400 == 0 ){
    leap_day = 1;
  }else{
    leap_day = 0;
  }

  if( dayofyear <= 31 ){ // January
    *month = 1;
    *date  = dayofyear;
  }else if( 32 <= dayofyear && dayofyear <= 59+leap_day ){ // February
    *month = 2;
    *date  = dayofyear - 31;
  }else if( 60+leap_day <= dayofyear && dayofyear <= 90+leap_day ){ // March
    *month = 3;
    *date  = dayofyear - 59-leap_day;
  }else if( 91+leap_day <= dayofyear && dayofyear <= 120+leap_day ){ // April
    *month = 4;
    *date  = dayofyear - 90-leap_day;
  }else if( 121+leap_day <= dayofyear && dayofyear <= 151+leap_day ){ // May
    *month = 5;
    *date  = dayofyear - 120-leap_day;
  }else if( 152+leap_day <= dayofyear && dayofyear <= 181+leap_day ){ // June
    *month = 6;
    *date  = dayofyear - 151-leap_day;
  }else if( 182+leap_day <= dayofyear && dayofyear <= 212+leap_day ){ // July
    *month = 7;
    *date  = dayofyear - 181-leap_day;
  }else if( 213+leap_day <= dayofyear && dayofyear <= 243+leap_day ){ // Aug
    *month = 8;
    *date  = dayofyear - 212-leap_day;
  }else if( 244+leap_day <= dayofyear && dayofyear <= 273+leap_day ){ // Sep
    *month = 9;
    *date  = dayofyear - 243-leap_day;
  }else if( 274+leap_day <= dayofyear && dayofyear <= 304+leap_day ){ // Oct
    *month = 10;
    *date  = dayofyear - 273-leap_day;
  }else if( 305+leap_day <= dayofyear && dayofyear <= 334+leap_day ){ // Nov
    *month = 11;
    *date  = dayofyear - 304-leap_day;
  }else if( 335+leap_day <= dayofyear ){ // December
    *month = 12;
    *date  = dayofyear - 334-leap_day;
  }else{
    printf("?dayofyear?\n");
    exit( 1 );
  }
}

/* -------------------------------------------------------------------
   ------------------------------------------------------------------- */
void read_sd_data( int argc, char **argv )
{
  int i, j, k, record, range;
  int32 sd_id, sds_id, n_datasets, n_file_attrs, index, sds_index;
  int32 dim_size[MAX_VAR_DIMS], start[MAX_VAR_DIMS];
  int32 rank, num_type, attributes, istat;
  char name[64];
  VOIDP  *v_limit;     /* generic pointer for void array    */

  /* Open the file and initiate the SD interface. */
  if( argc != 2 ){
    printf("argc != 2\n");
    printf("./cloud_sat_2B.v3 DATA/2007012134409_03775_CS_2B-GEOPROF_GRANULE_P_R04_E02.hdf\n");
    exit( 1 );
  }

  if ( (sd_id=SDstart( argv[1], DFACC_READ )) == FAIL ){
    printf ("nodata\n");
    exit( 1 );
  }

  /* Read the data  =========== */
  istat = SDfileinfo(sd_id, &n_datasets, &n_file_attrs);

  for( k = 0 ; k < n_datasets ; k++ ){
    sds_id = SDselect( sd_id, k );
    istat = SDgetinfo(sds_id, name, &rank, dim_size, &num_type, &attributes);

    // printf("%s\n", name );
    
    for( i = 0 ; i < rank ; i++ ){
      start[i] = 0;
    }

    /* ---------------------------------------------------------- */
    if( strcmp( name, "Radar_Reflectivity" ) == 0 ){
      Z_int = (short int **)malloc( sizeof( short int * )*dim_size[0] );
      Z_int[0] 
        = (short int *)malloc( sizeof( short int )*dim_size[0]*dim_size[1] );
      for( i = 1 ; i < dim_size[0] ; i++ )
        Z_int[i] = Z_int[0] + i * dim_size[1];

      istat = SDreaddata( sds_id, start, NULL, dim_size, Z_int[0] );
      istat = SDendaccess(sds_id);
    }
    /* ---------------------------------------------------------- */

    /* ---------------------------------------------------------- */
    if( strcmp( name, "Height" ) == 0 ){
      Max_record = dim_size[0];
      Max_range  = dim_size[1];

      Height = (short int **)malloc( sizeof( short int * )*dim_size[0] );
      Height[0] 
        = (short int *)malloc( sizeof( short int )*dim_size[0]*dim_size[1] );
      for( i = 1 ; i < dim_size[0] ; i++ )
        Height[i] = Height[0] + i * dim_size[1];

      istat = SDreaddata( sds_id, start, NULL, dim_size, Height[0] );
      istat = SDendaccess(sds_id);
    }
    /* ---------------------------------------------------------- */

  }
  /* End Determine the contents of the file. =================== */

  /* Terminate access to the SD interface and close the file. */
  istat = SDend(sd_id);
}

/* -------------------------------------------------------------------
   ------------------------------------------------------------------- */
void read_v_data( int argc, char **argv )
{
  int  nelem;
  int  k, m, i, record, range;
  int32  file_id;        /* input file identifier               */
  int32  istat;          /*  scratch integer mainly for HDF lib */
  int32  vdata_ref = -1; /* current Vdata reference             */
  int  vdata_count = 0;  /* counter for Vdata                   */
  char  *pc;                       /* generic pointer to chars  */
  int32  vdata_id;                 /* current Vdata identifier  */
  char  vdata_name[VSNAMELENMAX];  /* current Vdata name        */
  int32  n_records;            /* records in current Vdata      */
  int32  interlace;            /* interlace in current Vdata    */
  int32  vdata_size;           /* record size in current Vdata  */
  char  fields[CHAR_4_FIELDS]; /* string with current Vdata fields */
  int32  nfields;              /* number of expected fields  */
  int  vdata_order;            /* index in array vrefer      */
  int  size;                   /* size of current field in current Vdata */
  unsigned int record_size;    /* evaluated record size; should agree
				  with vdata_size    */
  int  field_order;
  int32  field_type;     /* type of current field in current Vdata */
  char  *databuf;        /* address of allocated memory  */
  FILE  *outfile;        /* default out file   */
  char  outname[120];    /* out file name      */
  
  long int           *intp;    /* pointer to unpacked arrays  */
  unsigned long int  *uintp;
  short int          *sintp;
  unsigned short int *usintp;
  double   *dp;
  float    *fp;
  char     *cp;
  uint8  *tinyunint;
  unsigned char  *ucp;
  VOIDP fldbufptrs[NFIELDS];    /* pointers to the unpacked aray(s) */

  void check_hdf_file(char *);    /* check consistency of hdf file  */
  
  /* -------------------------------------------------------------------
     check file, open it and start the interface 
     -------------------------------------------------------------------  */
  /* Open the file and initiate the SD interface. */
  if( argc != 2 ){
    printf("argc != 2\n");
    printf("./cloud_sat_2B Overshoot/2007012134409_03775_CS_2B-GEOPROF_GRANULE_P_R04_E02.hdf\n");
    exit( 1 );
  }

  if( (file_id = Hopen(argv[1],DFACC_READ,0)) == FAIL){
    printf("Hopen\n");
    exit( 1 );
  }
  
  if ( Vstart(file_id) == FAIL){
    printf("Vstart\n");
    exit( 1 );
  }
  
  /* ---------------------------------------------------------------
     start decodiing, one vdata at a time 
     ---------------------------------------------------------------  */
  
  vdata_ref   = -1;
  // an error occurs at vdata_order == 229 since a year of 2012.
  vdata_count = 200;
  for (vdata_order = 0; vdata_order < vdata_count ; vdata_order++) {
    /*
      a)  get the reference for the vdata
      b)  attach the vdata (on error exit)
      c)  get info on the vdata
      1) number of records
      2) interlace mode
      3) content
      4) size
      5) name
      6) number of fields
    */
    
    if ( (vdata_ref = VSgetid(file_id, vdata_ref)      ) == FAIL){
      printf("VSgetid\n");
      exit( 1 );
    }
    if ( (vdata_id  = VSattach(file_id, vdata_ref, "r")) == FAIL){
      printf("VSattach\n");
      exit( 1 );
    }
    if( VSinquire(vdata_id,&n_records,&interlace,fields,&vdata_size,vdata_name)
	== FAIL){
      printf("VSinquire\n");
      exit( 1 );
    }
    if ((nfields = VFnfields(vdata_id)) == FAIL){
      printf("VFnfields\n");
      exit( 1 );
    }
    
    /* print results to terminal         */
    /*
    printf("vdata_order = %d\n", vdata_order );
    printf("vdata_name  = %s\n", vdata_name  );
    printf("vdata_ref   = %d\n", vdata_ref   );
    printf("fields      = %s\n", fields      );
    printf("num. fields = %d\n\n",nfields    );
    */
    
    /* check that sufficient fields are allocated       */
    
    if (nfields >= NFIELDS){
      printf("nfields >= NFIELDS\n");
      exit( 1 );
    }
    
    /* -----------------------------------------------------------
       get info on fields
       a) size in file
       b) name
       c) size in memory
       
       adding all the memory sizes check that the result is consistent
       with what was expected for the record size
       ------------------------------------------------------------ */
    
    for (record_size = 0, i=0 ; i < nfields;i++) {   /* info on fields  */ 
      size = VFfieldisize(vdata_id,i);    /* size   */
      record_size += size;      /* update   */
    }

    // fprintf(stdout,"Record size %4d \n",record_size);
    if ( record_size != vdata_size){
      printf(" record_size != vdata_size \n");
      exit( 1 );
    }    
    
    /* ---------------------------------------------------------------
       preset program so that all fields will be loaded; 
       allocate core and read all records; 
       check that the count or records read is consistent with 
       what was expected
       --------------------------------------------------------------- */
    
    if ( VSsetfields(vdata_id, fields) == FAIL){
      printf("VSsetfields\n");
      exit( 1 );
    }
    
    if ((databuf = calloc((unsigned)n_records,(unsigned)vdata_size))
	== (char *)NULL) {
      fprintf(stdout,"Error in allocating databuf\n"); 
      exit( 1 );
    }
    
    if ( (istat = VSread(vdata_id, (VOIDP)databuf, n_records ,FULL_INTERLACE))
	 == FAIL){
      printf("VSread\n");
      exit( 1 );
    }
    
    // fprintf(stdout,"Records read %d\n",istat);
    if (istat != n_records) {
      fprintf(stdout,"Mismatch in records\n"); 
      exit( 1 );
    }

    /* ---------------------------------------------------------------
       decode one field at a time; each field will be sent to a different file;
       this just because I'm too lazy to exchange the order of for loops 
       ---------------------------------------------------------------- */
    for (field_order = 0; field_order < nfields; field_order++) {
      
      /* -----------------------------------------------------------
	 for each field we need to know:
	 
	 a) its type 
	 b) its size
	 c)  number of elements 
	 ------------------------------------------------------------ */
      
      if ((field_type = VFfieldtype(vdata_id,field_order)) == FAIL){
	printf("VFfieldtype\n");
	exit( 1 );
      }
      field_type = field_type & ( ~(DFNT_NATIVE | DFNT_LITEND));
      size = VFfieldesize(vdata_id,field_order);
      nelem = VFfieldorder(vdata_id,field_order);

      // =================================================================
      if( strcmp( vdata_name, "Latitude" ) == 0 ){
        if((fp = (float *)calloc((unsigned)n_records*nelem,sizeof(float)))
           == (float *)NULL){
          fprintf(stdout,"failed calloc index\n");
          exit( 1 );
        }
        fldbufptrs[0] = (VOIDP)fp;

        if ( VSfpack(vdata_id, _HDF_VSUNPACK, fields,
                     (VOIDP)databuf,(unsigned long)(vdata_size*n_records),
                     n_records, VFfieldname(vdata_id,field_order),fldbufptrs)
             == FAIL){
          printf("VSfpack\n");
          exit( 1 );
        }
        for ( record = 0; record < n_records ; record++ ){
          Latitude[record] = *fp++;
        }
      }

      // =================================================================
      if( strcmp( vdata_name, "Longitude" ) == 0 ){
        if((fp = (float *)calloc((unsigned)n_records*nelem,sizeof(float)))
           == (float *)NULL){
          fprintf(stdout,"failed calloc index\n");
          exit( 1 );
        }
        fldbufptrs[0] = (VOIDP)fp;

        if ( VSfpack(vdata_id, _HDF_VSUNPACK, fields,
                     (VOIDP)databuf,(unsigned long)(vdata_size*n_records),
                     n_records, VFfieldname(vdata_id,field_order),fldbufptrs)
             == FAIL){
          printf("VSfpack\n");
          exit( 1 );
        }
        for ( record = 0; record < n_records ; record++ ){
          Longitude[record] = *fp++;
        }
      }

      // =================================================================
      if( strcmp( vdata_name, "Profile_time" ) == 0 ){
        if((fp = (float *)calloc((unsigned)n_records*nelem,sizeof(float)))
           == (float *)NULL){
          fprintf(stdout,"failed calloc index\n");
          exit( 1 );
        }
        fldbufptrs[0] = (VOIDP)fp;

        if ( VSfpack(vdata_id, _HDF_VSUNPACK, fields,
                     (VOIDP)databuf,(unsigned long)(vdata_size*n_records),
                     n_records, VFfieldname(vdata_id,field_order),fldbufptrs)
             == FAIL){
          printf("VSfpack\n");
          exit( 1 );
        }
        for ( record = 0; record < n_records ; record++ ){
          Time[record] = *fp++;
        }
      }

      // =================================================================
      if( strcmp( vdata_name, "UTC_start" ) == 0 ){
        if((fp = (float *)calloc((unsigned)n_records*nelem,sizeof(float)))
           == (float *)NULL){
          fprintf(stdout,"failed calloc index\n");
          exit( 1 );
        }
        fldbufptrs[0] = (VOIDP)fp;

        if ( VSfpack(vdata_id, _HDF_VSUNPACK, fields,
                     (VOIDP)databuf,(unsigned long)(vdata_size*n_records),
                     n_records, VFfieldname(vdata_id,field_order),fldbufptrs)
             == FAIL){
          printf("VSfpack\n");
          exit( 1 );
        }
	Time_start = *fp++;
      }

      // =================================================================
      if( strcmp( vdata_name, "Navigation_land_sea_flag" ) == 0 ){
	if((tinyunint = (uint8 *)calloc(n_records*nelem,sizeof(int)))
	   == (uint8 *)NULL){
          fprintf(stdout,"failed calloc index\n");
          exit( 1 );
        }
        fldbufptrs[0] = (VOIDP)tinyunint;

        if ( VSfpack(vdata_id, _HDF_VSUNPACK, fields,
                     (VOIDP)databuf,(int)(vdata_size*n_records),
                     n_records, VFfieldname(vdata_id,field_order),fldbufptrs)
             == FAIL){
          printf("VSfpack\n");
          exit( 1 );
        }
        for( record = 0; record < n_records ; record++ ){
	  Land_sea_flag[record] = (short)*tinyunint++;
        }
      }
    }        /* close for on field_order */
    free(databuf);         /* release memory   */
  }

  (void) Vend(file_id);       /* close the interface  */
  (void) Hclose(file_id);       /* close the file   */
}

/* -------------------------------------------------------------------
   ------------------------------------------------------------------- */
void make_arrays( )
{
  int i;

  Latitude = (float32 *)malloc(Max_record * sizeof(float32));
  Longitude= (float32 *)malloc(Max_record * sizeof(float32));

  Time  = (float32 *)malloc(Max_record * sizeof(float32));
  Dayfrom1800y = (double *)malloc(Max_record * sizeof(double));
  Land_sea_flag = (short int *)malloc(Max_record * sizeof(short int));

  Z = (float32 **)malloc( sizeof( float32 * )*Max_record );
  Z[0] 
    = (float32 *)malloc( sizeof( float32 )*Max_record*Max_range );
  for( i = 1 ; i < Max_record ; i++ )
    Z[i] = Z[0] + i * Max_range;
}

/* -------------------------------------------------------------------
   ------------------------------------------------------------------- */
void calc_data( )
{
  int record, range, hour, min, sec, additional_day;
  double date;

  for( record = 0 ; record < Max_record ; record++ ){
    Time[record] += Time_start; // seconds from 00UTC

    Dayfrom1800y[record]
      = day_from_1800y( Year, Month, Date ) + Time[record] / 86400.;

    for ( range = 0; range < Max_range ; range++ ){
      Z[record][range] = Z_int[record][range] / 100.;
    }
  }
}

/* -------------------------------------------------------------------
   ------------------------------------------------------------------- */
void print_data( )
{
  int record, range;
  int year, month, date, hour, min, sec, secfromdate;
  char filename[100];
  FILE *fp;
  
  for( record = 0 ; record < Max_record ; record++ ){
    dayfrom1800y_2_yymmdd( &year, &month, &date, Dayfrom1800y[record] );

    secfromdate = ( Dayfrom1800y[record]-(int)(Dayfrom1800y[record]) ) * 86400;
    hour = (int)( secfromdate / 3600 );
    min  = (int)( ( secfromdate - hour * 3600 ) / 60. );
    sec  = (int)( secfromdate - hour*3600 - min*60 );
    
    set_file_name( year, month, date, filename );
    fp = fopen( filename, "a" ); // CHECK!!!,  "a" denotes "add"

    fprintf( fp, "%d %d %d  %d %d %d  %3.3f %3.3f  %d\n", 
	     year, 
	     month,
	     date,
	     hour,
	     min, 
	     sec, 
	     Latitude[record],
	     Longitude[record],
	     Max_range );

    for ( range = 0; range < Max_range ; range++ ){
      fprintf( fp, "%5.3f  %5.2f\n", 
	      Height[record][range] / 1000., 
	      Z[record][range] );
    }
    fclose( fp );
  }
}

/* -------------------------------------------------------------------
   ------------------------------------------------------------------- */
void set_file_name( int year, int month, int date, char buff[] )
{
  char yy_char[10], mm_char[10], dd_char[10];

  sprintf( yy_char, "%d", year );
  if( month < 10 )
    sprintf( mm_char, "%s%d", "0", month );
  else
    sprintf( mm_char, "%d", month );
  if( date < 10 )
    sprintf( dd_char, "%s%d", "0", date );
  else
    sprintf( dd_char, "%d", date );

  sprintf( buff, "%s.%s.%s.txt", // CHECK!!!
	     yy_char, mm_char, dd_char );
}

/* --------------------------------------------------------------------
   -------------------------------------------------------------------- */
double day_from_1800y( int year, int month, double date )
{
  int dayfrom1800y, year_int;

  if(       month ==  1 ){ dayfrom1800y =   0;
  }else if( month ==  2 ){ dayfrom1800y =  31;
  }else if( month ==  3 ){ dayfrom1800y =  59;
  }else if( month ==  4 ){ dayfrom1800y =  90;
  }else if( month ==  5 ){ dayfrom1800y = 120;
  }else if( month ==  6 ){ dayfrom1800y = 151;
  }else if( month ==  7 ){ dayfrom1800y = 181;
  }else if( month ==  8 ){ dayfrom1800y = 212;
  }else if( month ==  9 ){ dayfrom1800y = 243;
  }else if( month == 10 ){ dayfrom1800y = 273;
  }else if( month == 11 ){ dayfrom1800y = 304;
  }else if( month == 12 ){ dayfrom1800y = 334;
  }else{
    printf("wrong month data\n");
    exit( 1 );
  }

  for( year_int = 1800 ; year_int < year ; year_int++ ){
    dayfrom1800y += 365;
    if( year_int%4 == 0 && year_int%100 != 0 && year_int%400 != 0 ){
      dayfrom1800y += 1;
    }else if( year_int%4 == 0 && year_int%100 == 0 && year_int%400 != 0 ){
    }else if( year_int%4 == 0 && year_int%400 == 0 ){
      dayfrom1800y += 1;
    }
  }

  if( 3 <= month ){
    if( year%4 == 0 && year%100 != 0 && year%400 != 0 ){
      dayfrom1800y += 1;
    }else if( year%4 == 0 && year%100 == 0 && year%400 != 0 ){
    }else if( year%4 == 0 && year%400 == 0 ){
      dayfrom1800y += 1;
    }
  }

  return dayfrom1800y + date; // 1 = Jan./01, 1800
}

/* --------------------------------------------------------------------
   -------------------------------------------------------------------- */
void dayfrom1800y_2_yymmdd( int *year, int *month, int *date, 
                            double dayfrom1800y )
{
  int year_int, dayfrom1800y_int, jan1_dayfrom1800y, dayofyear, leap_day;

  dayfrom1800y_int = (int)( dayfrom1800y );

  jan1_dayfrom1800y = 1;
  for( year_int = 1800 ; year_int < 2030 ; year_int++ ){
    if( dayfrom1800y_int >= jan1_dayfrom1800y ){
      // cout<< dayfrom1800y_int <<"  "<< jan1_dayfrom1800y <<endl;
      *year = year_int;
      dayofyear = dayfrom1800y_int - jan1_dayfrom1800y + 1;
    }else{
      break;
    }

    if( year_int%4 == 0 && year_int%100 != 0 && year_int%400 != 0 ){
      jan1_dayfrom1800y += 366;
    }else if( year_int%4 == 0 && year_int%400 == 0 ){
      jan1_dayfrom1800y += 366;
    }else{
      jan1_dayfrom1800y += 365;
    }
  }

  if( *year%4 == 0 && *year%100 != 0 && *year%400 != 0 ){
    leap_day = 1;
  }else if( *year%4 == 0 && *year%400 == 0 ){
    leap_day = 1;
  }else{
    leap_day = 0;
  }

  if( dayofyear <= 31 ){ // January
    *month = 1;
    *date  = dayofyear;
  }else if( 32 <= dayofyear && dayofyear <= 59+leap_day ){ // February
    *month = 2;
    *date  = dayofyear - 31;
  }else if( 60+leap_day <= dayofyear && dayofyear <= 90+leap_day ){ // March
    *month = 3;
    *date  = dayofyear - 59-leap_day;
  }else if( 91+leap_day <= dayofyear && dayofyear <= 120+leap_day ){ // April
    *month = 4;
    *date  = dayofyear - 90-leap_day;
  }else if( 121+leap_day <= dayofyear && dayofyear <= 151+leap_day ){ // May
    *month = 5;
    *date  = dayofyear - 120-leap_day;
  }else if( 152+leap_day <= dayofyear && dayofyear <= 181+leap_day ){ // June
    *month = 6;
    *date  = dayofyear - 151-leap_day;
  }else if( 182+leap_day <= dayofyear && dayofyear <= 212+leap_day ){ // July
    *month = 7;
    *date  = dayofyear - 181-leap_day;
  }else if( 213+leap_day <= dayofyear && dayofyear <= 243+leap_day ){ // Aug
    *month = 8;
    *date  = dayofyear - 212-leap_day;
  }else if( 244+leap_day <= dayofyear && dayofyear <= 273+leap_day ){ // Sep
    *month = 9;
    *date  = dayofyear - 243-leap_day;
  }else if( 274+leap_day <= dayofyear && dayofyear <= 304+leap_day ){ // Oct
    *month = 10;
    *date  = dayofyear - 273-leap_day;
  }else if( 305+leap_day <= dayofyear && dayofyear <= 334+leap_day ){ // Nov
    *month = 11;
    *date  = dayofyear - 304-leap_day;
  }else if( 335+leap_day <= dayofyear ){ // December
    *month = 12;
    *date  = dayofyear - 334-leap_day;
  }else{
    printf("?dayofyear?\n");
    exit( 1 );
  }
}

/* -------------------------------------------------------------------
   ------------------------------------------------------------------- */
void free_arrays( )
{
  // 1D -------------------
  free( Latitude );
  free( Longitude );
  free( Time );
  free( Land_sea_flag );
  free( Dayfrom1800y );

  // 2D ------------------
  free(Z_int[0]);
  free(Z_int);
  free(Height[0]);
  free(Height);
  free(Z[0]);
  free(Z);
}
