/**********************************************************************/
/*                                                                    */
/* This program reads CALIPSO data, Total_Attenuated_Backscatter_532. */
/*                                                                    */
/**********************************************************************/

#include "cloud_sat_2B.h"

/* -------------------------------------------------------------------
   ------------------------------------------------------------------- */
main( int argc, char **argv ) 
{
  // option( argc, argv );
  
  read_year_month_day( argc, argv );

  read_sd_data( argc, argv );
  read_v_data(  argc, argv );

  calc_data( );

  // ============================
  // print_land_sea_flg( );
  // print_check_data( );

  // print_average_data( );
  // print_data( );
  print_hokan_data( );
}


/* -------------------------------------------------------------------
   ------------------------------------------------------------------- */
void read_year_month_day( int argc, char **argv )
{
  char c1, c2, c3, c4;
  char y1[4], y2, y3[1], *y4, d1, d2, d3;
  int year, day;

  // Overshoot/2007012134409_03775_CS_2B-GEOPROF_GRANULE_P_R04_E02.hdf

  sscanf( argv[1], "%*8c%4d%3d%2d%2d%2d", 
	  &Year, &Julian_day, &Hour_start, &Minute_start, &Sec_start);

  if( !( Year == 2006 || Year == 2007 ) ){
    printf("!( Year == 2006 || Year == 2007 )\n");
    exit( 1 );
  }

  julian_2_month_day( );
  /*
  printf("#  %d, %d, %d, %d, %d; %d  %d\n",
	 Year, 
	 Julian_day, 
	 Hour_start, 
	 Minute_start, 
	 Sec_start,
	 Month,
	 Date );
  exit( 1 );
  */
}

/* ------------------------------------------------------------------------
   ------------------------------------------------------------------------ */
void julian_2_month_day( )
{
  if( Julian_day <= 31 ){ // January
    Month = 1;
    Date  = Julian_day;
  }else if( 32 <= Julian_day && Julian_day <= 59 ){ // February
    Month = 2;
    Date  = Julian_day - 31;
  }else if( 60 <= Julian_day && Julian_day <= 90 ){ // March
    Month = 3;
    Date  = Julian_day - 59;
  }else if( 91 <= Julian_day && Julian_day <= 120 ){ // April
    Month = 4;
    Date  = Julian_day - 90;
  }else if( 121 <= Julian_day && Julian_day <= 151 ){ // May
    Month = 5;
    Date  = Julian_day - 120;
  }else if( 152 <= Julian_day && Julian_day <= 181 ){ // June
    Month = 6;
    Date  = Julian_day - 151;
  }else if( 182 <= Julian_day && Julian_day <= 212 ){ // July
    Month = 7;
    Date  = Julian_day - 181;
  }else if( 213 <= Julian_day && Julian_day <= 243 ){ // August
    Month = 8;
    Date  = Julian_day - 212;
  }else if( 244 <= Julian_day && Julian_day <= 273 ){ // September
    Month = 9;
    Date  = Julian_day - 243;
  }else if( 274 <= Julian_day && Julian_day <= 304 ){ // October
    Month = 10;
    Date  = Julian_day - 273;
  }else if( 305 <= Julian_day && Julian_day <= 334 ){ // November
    Month = 11;
    Date  = Julian_day - 304;
  }else if( 335 <= Julian_day ){ // December
    Month = 12;
    Date  = Julian_day - 334;
  }else{
    printf("?? %d\n", Julian_day );
    exit( 1 );
  }
}

/* -------------------------------------------------------------------
   ------------------------------------------------------------------- */
void read_sd_data( int argc, char **argv )
{
  int i, j, k, record, range;
  int32 sd_id, sds_id, n_datasets, n_file_attrs, index, sds_index;
  int32 dim_size[MAX_VAR_DIMS], start[MAX_VAR_DIMS];
  int32 rank, num_type, attributes, istat;
  char name[64];
  VOIDP  *v_limit;     /* generic pointer for void array    */

  /* Open the file and initiate the SD interface. */
  if( argc != 2 ){
    printf("argc != 2\n");
    printf("./cloud_sat_2B Overshoot/2007012134409_03775_CS_2B-GEOPROF_GRANULE_P_R04_E02.hdf\n");
    exit( 1 );
  }

  if ( (sd_id=SDstart( argv[1], DFACC_READ )) == FAIL ){
    printf ("nodata\n");
    exit( 1 );
  }

  /* Read the data  =========== */
  istat = SDfileinfo(sd_id, &n_datasets, &n_file_attrs);

  for( k = 0 ; k < n_datasets ; k++ ){
    sds_id = SDselect( sd_id, k );
    istat = SDgetinfo(sds_id, name, &rank, dim_size, &num_type, &attributes);

    // printf("%s\n", name );
    
    for( i = 0 ; i < rank ; i++ ){
      start[i] = 0;
    }

    /* ---------------------------------------------------------- */
    if( strcmp( name, "Height" ) == 0 ){
      Max_record = dim_size[0];
      Max_range  = dim_size[1];

      if( Max_record > MAX_RECORD || Max_range != MAX_RANGE ){
	printf("record or range error, %d  %d\n", 
	       Max_record, Max_range );
	exit( 1 );
      }
      istat = SDreaddata( sds_id, start, NULL, dim_size, Height );
    }
    istat = SDendaccess(sds_id);
    /* ---------------------------------------------------------- */

    /* ---------------------------------------------------------- */
    if( strcmp( name, "Radar_Reflectivity" ) == 0 ){
      if( dim_size[0] != Max_record || dim_size[1] != Max_range ){
	printf("record or range error, %d  %d\n", 
	       Max_record, Max_range );
	exit( 1 );
      }
      istat = SDreaddata( sds_id, start, NULL, dim_size, Z_int );
    }
    istat = SDendaccess(sds_id);
    /* ---------------------------------------------------------- */

  }
  /* End Determine the contents of the file. =================== */

  /* Terminate access to the SD interface and close the file. */
  istat = SDend(sd_id);
}

/* -------------------------------------------------------------------
   ------------------------------------------------------------------- */
void read_v_data( int argc, char **argv )
{
  int  nelem;
  int  k, m, i, record, range;
  int32  file_id;        /* input file identifier               */
  int32  istat;          /*  scratch integer mainly for HDF lib */
  int32  vdata_ref = -1; /* current Vdata reference             */
  int  vdata_count = 0;  /* counter for Vdata                   */
  char  *pc;                       /* generic pointer to chars  */
  int32  vdata_id;                 /* current Vdata identifier  */
  char  vdata_name[VSNAMELENMAX];  /* current Vdata name        */
  int32  n_records;            /* records in current Vdata      */
  int32  interlace;            /* interlace in current Vdata    */
  int32  vdata_size;           /* record size in current Vdata  */
  char  fields[CHAR_4_FIELDS]; /* string with current Vdata fields */
  int32  nfields;              /* number of expected fields  */
  int  vdata_order;            /* index in array vrefer      */
  int  size;                   /* size of current field in current Vdata */
  unsigned int record_size;    /* evaluated record size; should agree
				  with vdata_size    */
  int  field_order;
  int32  field_type;     /* type of current field in current Vdata */
  char  *databuf;        /* address of allocated memory  */
  FILE  *outfile;        /* default out file   */
  char  outname[120];    /* out file name      */
  
  long int           *intp;    /* pointer to unpacked arrays  */
  unsigned long int  *uintp;
  short int          *sintp;
  unsigned short int *usintp;
  double   *dp;
  float    *fp;
  char     *cp;
  uint8  *tinyunint;
  unsigned char  *ucp;
  VOIDP fldbufptrs[NFIELDS];    /* pointers to the unpacked aray(s) */

  void check_hdf_file(char *);    /* check consistency of hdf file  */
  
  /* -------------------------------------------------------------------
     check file, open it and start the interface 
     -------------------------------------------------------------------  */
  /* Open the file and initiate the SD interface. */
  if( argc != 2 ){
    printf("argc != 2\n");
    printf("./cloud_sat_2B Overshoot/2007012134409_03775_CS_2B-GEOPROF_GRANULE_P_R04_E02.hdf\n");
    exit( 1 );
  }

  if( (file_id = Hopen(argv[1],DFACC_READ,0)) == FAIL){
    printf("Hopen\n");
    exit( 1 );
  }
  
  if ( Vstart(file_id) == FAIL){
    printf("Vstart\n");
    exit( 1 );
  }
  
  /* ---------------------------------------------------------------
     start decodiing, one vdata at a time 
     ---------------------------------------------------------------  */
  
  vdata_ref   = -1;
  vdata_count = 405;
  for (vdata_order = 0; vdata_order < vdata_count ; vdata_order++) {
    /*
      a)  get the reference for the vdata
      b)  attach the vdata (on error exit)
      c)  get info on the vdata
      1) number of records
      2) interlace mode
      3) content
      4) size
      5) name
      6) number of fields
    */
    
    if ( (vdata_ref = VSgetid(file_id, vdata_ref)      ) == FAIL){
      printf("VSgetid\n");
      exit( 1 );
    }
    if ( (vdata_id  = VSattach(file_id, vdata_ref, "r")) == FAIL){
      printf("VSattach\n");
      exit( 1 );
    }
    if( VSinquire(vdata_id,&n_records,&interlace,fields,&vdata_size,vdata_name)
	== FAIL){
      printf("VSinquire\n");
      exit( 1 );
    }
    if ((nfields = VFnfields(vdata_id)) == FAIL){
      printf("VFnfields\n");
      exit( 1 );
    }
    
    /* print results to terminal         */
    /*
    printf("vdata_order = %d\n", vdata_order );
    printf("vdata_name  = %s\n", vdata_name  );
    printf("vdata_ref   = %d\n", vdata_ref   );
    printf("fields      = %s\n", fields      );
    printf("num. fields = %d\n\n",nfields    );
    */
    
    /* check that sufficient fields are allocated       */
    
    if (nfields >= NFIELDS){
      printf("nfields >= NFIELDS\n");
      exit( 1 );
    }
    
    /* -----------------------------------------------------------
       get info on fields
       a) size in file
       b) name
       c) size in memory
       
       adding all the memory sizes check that the result is consistent
       with what was expected for the record size
       ------------------------------------------------------------ */
    
    for (record_size = 0, i=0 ; i < nfields;i++) {   /* info on fields  */ 
      size = VFfieldisize(vdata_id,i);    /* size   */
      record_size += size;      /* update   */
    }

    // fprintf(stdout,"Record size %4d \n",record_size);
    if ( record_size != vdata_size){
      printf(" record_size != vdata_size \n");
      exit( 1 );
    }    
    
    /* ---------------------------------------------------------------
       preset program so that all fields will be loaded; 
       allocate core and read all records; 
       check that the count or records read is consistent with 
       what was expected
       --------------------------------------------------------------- */
    
    if ( VSsetfields(vdata_id, fields) == FAIL){
      printf("VSsetfields\n");
      exit( 1 );
    }
    
    if ((databuf = calloc((unsigned)n_records,(unsigned)vdata_size))
	== (char *)NULL) {
      fprintf(stdout,"Error in allocating databuf\n"); 
      exit( 1 );
    }
    
    if ( (istat = VSread(vdata_id, (VOIDP)databuf, n_records ,FULL_INTERLACE))
	 == FAIL){
      printf("VSread\n");
      exit( 1 );
    }
    
    // fprintf(stdout,"Records read %d\n",istat);
    if (istat != n_records) {
      fprintf(stdout,"Mismatch in records\n"); 
      exit( 1 );
    }

    /* ---------------------------------------------------------------
       decode one field at a time; each field will be sent to a different file;
       this just because I'm too lazy to exchange the order of for loops 
       ---------------------------------------------------------------- */
    for (field_order = 0; field_order < nfields; field_order++) {
      
      /* -----------------------------------------------------------
	 for each field we need to know:
	 
	 a) its type 
	 b) its size
	 c)  number of elements 
	 ------------------------------------------------------------ */
      
      if ((field_type = VFfieldtype(vdata_id,field_order)) == FAIL){
	printf("VFfieldtype\n");
	exit( 1 );
      }
      field_type = field_type & ( ~(DFNT_NATIVE | DFNT_LITEND));
      size = VFfieldesize(vdata_id,field_order);
      nelem = VFfieldorder(vdata_id,field_order);

      // =================================================================
      if( strcmp( vdata_name, "Latitude" ) == 0 ){
        if((fp = (float *)calloc((unsigned)n_records*nelem,sizeof(float)))
           == (float *)NULL){
          fprintf(stdout,"failed calloc index\n");
          exit( 1 );
        }
        fldbufptrs[0] = (VOIDP)fp;

        if ( VSfpack(vdata_id, _HDF_VSUNPACK, fields,
                     (VOIDP)databuf,(unsigned long)(vdata_size*n_records),
                     n_records, VFfieldname(vdata_id,field_order),fldbufptrs)
             == FAIL){
          printf("VSfpack\n");
          exit( 1 );
        }

        if( n_records != Max_record || nelem != 1 ){
          printf("error at Latitude\n");
          exit( 1 );
        }
        for ( record = 0; record < n_records ; record++ ){
          Latitude[record] = *fp++;
        }
      }

      // =================================================================
      if( strcmp( vdata_name, "Longitude" ) == 0 ){
        if((fp = (float *)calloc((unsigned)n_records*nelem,sizeof(float)))
           == (float *)NULL){
          fprintf(stdout,"failed calloc index\n");
          exit( 1 );
        }
        fldbufptrs[0] = (VOIDP)fp;

        if ( VSfpack(vdata_id, _HDF_VSUNPACK, fields,
                     (VOIDP)databuf,(unsigned long)(vdata_size*n_records),
                     n_records, VFfieldname(vdata_id,field_order),fldbufptrs)
             == FAIL){
          printf("VSfpack\n");
          exit( 1 );
        }

        if( n_records != Max_record || nelem != 1 ){
          printf("error at Longitude\n");
          exit( 1 );
        }
        for ( record = 0; record < n_records ; record++ ){
          Longitude[record] = *fp++;
        }
      }

      // =================================================================
      if( strcmp( vdata_name, "Profile_time" ) == 0 ){
        if((fp = (float *)calloc((unsigned)n_records*nelem,sizeof(float)))
           == (float *)NULL){
          fprintf(stdout,"failed calloc index\n");
          exit( 1 );
        }
        fldbufptrs[0] = (VOIDP)fp;

        if ( VSfpack(vdata_id, _HDF_VSUNPACK, fields,
                     (VOIDP)databuf,(unsigned long)(vdata_size*n_records),
                     n_records, VFfieldname(vdata_id,field_order),fldbufptrs)
             == FAIL){
          printf("VSfpack\n");
          exit( 1 );
        }

        if( n_records != Max_record || nelem != 1 ){
          printf("error at Profile_time\n");
          exit( 1 );
        }
        for ( record = 0; record < n_records ; record++ ){
          Time[record] = *fp++;
        }
      }

      // =================================================================
      if( strcmp( vdata_name, "UTC_start" ) == 0 ){
        if((fp = (float *)calloc((unsigned)n_records*nelem,sizeof(float)))
           == (float *)NULL){
          fprintf(stdout,"failed calloc index\n");
          exit( 1 );
        }
        fldbufptrs[0] = (VOIDP)fp;

        if ( VSfpack(vdata_id, _HDF_VSUNPACK, fields,
                     (VOIDP)databuf,(unsigned long)(vdata_size*n_records),
                     n_records, VFfieldname(vdata_id,field_order),fldbufptrs)
             == FAIL){
          printf("VSfpack\n");
          exit( 1 );
        }

        if( n_records != 1 || nelem != 1 ){
          printf("error at UTC_time\n");
          exit( 1 );
        }
	Time_start = *fp++;
      }

      // =================================================================
      if( strcmp( vdata_name, "Navigation_land_sea_flag" ) == 0 ){
	if((tinyunint = (uint8 *)calloc(n_records*nelem,sizeof(int)))
	   == (uint8 *)NULL){
          fprintf(stdout,"failed calloc index\n");
          exit( 1 );
        }
        fldbufptrs[0] = (VOIDP)tinyunint;

        if ( VSfpack(vdata_id, _HDF_VSUNPACK, fields,
                     (VOIDP)databuf,(int)(vdata_size*n_records),
                     n_records, VFfieldname(vdata_id,field_order),fldbufptrs)
             == FAIL){
          printf("VSfpack\n");
          exit( 1 );
        }
        if( n_records != Max_record || nelem != 1 ){
          printf("error at Navigation_land_sea_flag\n");
          exit( 1 );
        }
        for ( record = 0; record < n_records ; record++ ){
	  Land_sea_flag[record] = (short)*tinyunint++;
        }
      }
    }        /* close for on field_order */
    free(databuf);         /* release memory   */
  }

  (void) Vend(file_id);       /* close the interface  */
  (void) Hclose(file_id);       /* close the file   */
}

/* -------------------------------------------------------------------
   ------------------------------------------------------------------- */
void calc_data( )
{
  int record, range;

  for( record = 0 ; record < Max_record ; record++ ){
    Time[record] += Time_start;

    for ( range = 0; range < Max_range ; range++ ){
      Z[record][range] = Z_int[record][range] / 100.;
    }
    // exit( 1 );
  }
}

/* -------------------------------------------------------------------
   print Land_sea_flag[record] for mismo
   ------------------------------------------------------------------- */
void print_land_sea_flg( )
{
  int record, range, record_init, record_count;

  for( record = 0 ; record < Max_record ; record+=10 ){
    if( -3 <= Latitude[record]  && Latitude[record]  <=  7 &&
        71 <= Longitude[record] && Longitude[record] <= 83 &&
	Land_sea_flag[record] != SEA_FLG ){

      printf("%5.2f  %5.2f  %d \n", 
	     Latitude[record],
	     Longitude[record],
	     Land_sea_flag[record] );
    }
  }
}

/* -------------------------------------------------------------------
   ------------------------------------------------------------------- */
void print_check_data( )
{
  int record, range, record_init, record_count;
  int hour, min;
  double sec, day;

  for( record = 0 ; record < Max_record ; record+=500 ){
    hour = (int)( Time[record] / 3600 );
    min  = (int)( ( Time[record] - hour * 3600 ) / 60. );
    sec  = Time[record] - hour*3600 - min*60;

    day = Date + hour/24.+min/24./60.;

    printf("%d  %f  %f  %f \n", 
	   Month, 
	   day, 
	   Latitude[record],
	   Longitude[record] );
  }
}

/* -------------------------------------------------------------------
   ------------------------------------------------------------------- */
double average_data( int record_av, int range_av )
{
  int record, range;
  int sum_num;
  double dbz;

  // ======================================================================
  dbz     = 0;
  sum_num = 0;

  for( record = 0 ; record < Max_record ; record++ ){
    if( Latitude_av[record_av]-Delta_lat_half <= Latitude[record] &&
	Latitude[record] < Latitude_av[record_av]+Delta_lat_half ){

      for( range = 0; range < Max_range ; range++ ){
	if( Altitude_av[range_av]-Delta_alt_half <= Height[record][range] &&
	    Height[record][range] < Altitude_av[range_av]+Delta_alt_half ){

	  dbz += Z[record][range];
	  sum_num += 1;

	}
      }
    }
  }
  // ======================================================================

  if( sum_num == 0 ){
    return -999;
  }else{
    dbz /= sum_num;
  }
  return dbz;
}

/* -------------------------------------------------------------------
   ------------------------------------------------------------------- */
double interpolate_data( int record_av, int range_av )
{
  int record, range;
  double dbz, z1, z2, z;

  // ======================================================================
  for( record = 0 ; record < Max_record-1 ; record++ ){
    if( ( Latitude[record] <= Latitude_av[record_av] &&
	  Latitude_av[record_av] < Latitude[record+1] ) ||
	( Latitude[record+1] <= Latitude_av[record_av] &&
	  Latitude_av[record_av] < Latitude[record] ) ){

      for( range = 0 ; range < Max_range-1 ; range++ ){
	if( Height[record][range+1] <= Altitude_av[range_av] &&
	    Altitude_av[range_av] < Height[record][range] ){
	  z1
	    = ( Z[record][range+1] - Z[record][range] ) 
	    / ( Height[record][range+1] - Height[record][range] )
	    * ( Altitude_av[range_av] - Height[record][range] ) 
	    + Z[record][range];
	}
      }

      for( range = 0 ; range < Max_range-1 ; range++ ){
	if( Height[record+1][range+1] <= Altitude_av[range_av] &&
	    Altitude_av[range_av] < Height[record+1][range] ){
	  z2
	    = ( Z[record+1][range+1] - Z[record+1][range] ) 
	    / ( Height[record+1][range+1] - Height[record+1][range] )
	    * ( Altitude_av[range_av] - Height[record+1][range] ) 
	    + Z[record+1][range];
	}
      }

      z
	= ( z2 - z1 ) / ( Latitude[record+1] - Latitude[record] ) 
	* ( Latitude_av[record_av] - Latitude[record] ) 
	+ z1;
      return z;
    }
  }
  return -999;
}

/* -------------------------------------------------------------------
   ------------------------------------------------------------------- */
void print_average_data( )
{
  int record_av, range_av;

  for( record_av = 0 ; record_av < Max_record_av ; record_av++ ){
    for( range_av = 0; range_av < Max_range_av ; range_av++ ){
      printf("%5.2f  %5.2f  %5.2f\n",
	     Latitude_av[record_av], 
	     Altitude_av[range_av],
	     interpolate_data( record_av, range_av ) );
    }
  }

}

/* -------------------------------------------------------------------
   ------------------------------------------------------------------- */
void print_data( )
{
  int record, range, record_init, record_count;
  int hour, min;
  double sec, dbz;

  // for( record = 0 ; record < Max_record ; record++ ){
  for( record = 0 ; record < Max_record ; record+=1 ){
    hour = (int)( Time[record] / 3600 );
    min  = (int)( ( Time[record] - hour * 3600 ) / 60. );
    sec  = Time[record] - hour*3600 - min*60;

    for ( range = 0; range < Max_range ; range++ ){

      if( Height[record][range] > 21000 ){
	continue;
      }

#if _OVERSHOOTING_
      if( Latitude[record]  < -10 ||   0 < Latitude[record]  ||
	  Longitude[record] < 177 || 180 < Longitude[record] ){
	continue;
      }
      printf("%5.3f  %5.3f  %g\n", 
	     Latitude[record],
	     Height[record][range] / 1000., 
	     Z[record][range] );
#endif

      /*
	printf("%d  %d  %d  %f  %f  %f  %g\n", 
	hour,
	min, 
	(int)( sec ), 
	Latitude[record],
	Longitude[record],
	altitude,
	dbz );
      */
    }
  }
}

/* -------------------------------------------------------------------
   ------------------------------------------------------------------- */
void print_hokan_data( )
{
  const int MAX_RANGE_HOKAN = 201;
  int record, range, range_hokan;
  int hour, min;
  double sec;
  double alt_hokan[MAX_RANGE_HOKAN], dz, dh;
  double z_hokan[MAX_RANGE_HOKAN];

  for( range_hokan = 0 ; range_hokan < MAX_RANGE_HOKAN ; range_hokan++ ){
    alt_hokan[range_hokan] = 100 * range_hokan; // [m]
  }

  for( record = 0 ; record < Max_record ; record++ ){
#if 0
    if( Latitude[record]  < -10 ||   0 < Latitude[record]  ||
	Longitude[record] < 177 || 180 < Longitude[record] ){
      continue;
    }
#endif
    hour = (int)( Time[record] / 3600 );
    min  = (int)( ( Time[record] - hour * 3600 ) / 60. );
    sec  = Time[record] - hour*3600 - min*60;

    for ( range_hokan = 0; range_hokan < MAX_RANGE_HOKAN ; range_hokan++ ){
      for ( range = 0; range < Max_range-1 ; range++ ){
	if( ( Height[record][range]   < alt_hokan[range_hokan] &&
	      Height[record][range+1] < alt_hokan[range_hokan] ) ||
	    ( Height[record][range]   > alt_hokan[range_hokan] &&
	      Height[record][range+1] > alt_hokan[range_hokan] ) ){
	  continue;
	}
	dz = Z[record][range+1] - Z[record][range];
	dh = Height[record][range+1] - Height[record][range];

	z_hokan[range_hokan] 
	  = Z[record][range]
	  + dz/dh*(alt_hokan[range_hokan]-Height[record][range]);
      }
      printf("%d %d %2.1f  %5.3f %5.3f  %5.3f %5.3f\n", 
	     hour, 
	     min,
	     sec,
	     Latitude[record],
	     Longitude[record],
	     alt_hokan[range_hokan]/1e3, 
	     z_hokan[range_hokan] );
      
    }
  }
}
